;;;;
;;;; ACL-COMPAT - EXCL
;;;;
;;;; This is a modified version of Chris Doubles ACL excl wrapper library
;;;; As stated in the changelogs of his original this file includes the 
;;;; IF* macro placed in the public domain by John Foderaro. 
;;;; See: http://www.franz.com/~jkf/ifstar.txt
;;;;
;;;; It is not clear to this point if future releases will lead to a combined
;;;; effort - So you may find newer versions of *this* file at 
;;;; http://www.dataheaven.de
;;;;

;;;; This is the header of Chris Doubles original file. (but without Changelog)
;;;;
;;;; ACL excl wrapper library for Corman Lisp - Version 1.1
;;;;
;;;; Copyright (C) 2000 Christopher Double. All Rights Reserved.

;(require 'nregex)

(in-package :acl-compat.excl)

(defvar if*-keyword-list '("then" "thenret" "else" "elseif"))

(defmacro if* (&rest args)
   (do ((xx (reverse args) (cdr xx))
	(state :init)
	(elseseen nil)
	(totalcol nil)
	(lookat nil nil)
	(col nil))
       ((null xx)
	(cond ((eq state :compl)
	       `(cond ,@totalcol))
	      (t (error "if*: illegal form ~s" args))))
       (cond ((and (symbolp (car xx))
		   (member (symbol-name (car xx))
			   if*-keyword-list
			   :test #'string-equal))
	      (setq lookat (symbol-name (car xx)))))

       (cond ((eq state :init)
	      (cond (lookat (cond ((string-equal lookat "thenret")
				   (setq col nil
					 state :then))
				  (t (error
				      "if*: bad keyword ~a" lookat))))
		    (t (setq state :col
			     col nil)
		       (push (car xx) col))))
	     ((eq state :col)
	      (cond (lookat
		     (cond ((string-equal lookat "else")
			    (cond (elseseen
				   (error
				    "if*: multiples elses")))
			    (setq elseseen t)
			    (setq state :init)
			    (push `(t ,@col) totalcol))
			   ((string-equal lookat "then")
			    (setq state :then))
			   (t (error "if*: bad keyword ~s"
					      lookat))))
		    (t (push (car xx) col))))
	     ((eq state :then)
	      (cond (lookat
		     (error
		      "if*: keyword ~s at the wrong place " (car xx)))
		    (t (setq state :compl)
		       (push `(,(car xx) ,@col) totalcol))))
	     ((eq state :compl)
	      (cond ((not (string-equal lookat "elseif"))
		     (error "if*: missing elseif clause ")))
	      (setq state :init)))))
#|
(defvar *initial-terminal-io* *terminal-io*)
(defvar *cl-default-special-bindings* nil)

(defun filesys-size (stream)
	(file-length stream))

(defun filesys-write-date (stream)
	(file-write-date stream))

#+obsolete
(defun stream-input-fn (stream)
  stream)

(defmethod stream-input-fn ((stream stream))
  stream)
	

(defun match-regexp (pattern string &key (return :string))
  (let ((res (cond ((stringp pattern)
		    (regex pattern string))
		   ((functionp pattern) (funcall pattern string))
		   (t (error "Wrong type for pattern")))))
    (case return
      (:string
       (values-list (cons (not (null res))
                          res)))
      (:index (error "REGEXP: INDEX Not implemented"))
      (otherwise (not (null res))))))

(defun compile-regexp (regexp)
  (compile nil (regex-compile regexp)))
|#
(defvar *current-case-mode* :case-insensitive-upper)

(defun intern* (s len package)
	(intern (string-upcase (subseq s 0 len)) package))
#|
(defun filesys-type (file-or-directory-name)
	(if (lw::file-directory-p file-or-directory-name)
		:directory
		(if (probe-file file-or-directory-name)
			:file
			nil)))
|#
(defmacro errorset (form &optional (announce nil) (catch-breaks nil))
  "This macro is incomplete.  It was hacked to get AllegroServe
running, but the announce and catch-breaks arguments are ignored.  See
documentation at
http://franz.com/support/documentation/6.1/doc/pages/operators/excl/errorset.htm
An implementation of the catch-breaks argument will necessarily be
implementation-dependent, since Ansi does not allow any
program-controlled interception of a break."
  (declare (ignore announce catch-breaks))
  `(let* ((ok nil)
          (results (ignore-errors
                     (prog1 (multiple-value-list ,form)
                       (setq ok t)))))
     (if ok
         (apply #'values t results)
         nil)))

#|
(defmacro atomically (&body forms)
  `(mp:without-preemption ,@forms))

(defmacro fast (&body forms)
  `(locally (declare (optimize (speed 3) (safety 0) (debug 0)))
	    ,@forms))

(defmacro without-package-locks (&body forms)
  `(progn ,@forms))
|#

(defun string-to-octets (string &key (null-terminate t) (start 0) end
                                     mb-vector make-mb-vector?
                                     (external-format :default))
  "This function returns a lisp-usb8-vector and the number of bytes copied."
  ;; The end parameter is different in ACL's lambda list, but this
  ;; variant lets us give an argument :end nil explicitly, and the
  ;; right thing will happen
  ;; LW: For base-string, this is the analog of ef:encode-lisp-string
  ;; NB: For now, ignore Newline-crlf translation
  (unless end (setf end (length string)))
  (when (eq external-format :default)
    (setq external-format (if (lw:text-string-p string)
                              #+win32 win32:*multibyte-code-page-ef*
                              #-win32 stream::*default-external-format*)))
  ;  (:default (if (base-string-p string) :latin-1 :unicode)
  (let* ((code-page (when (consp external-format)
                      (EF::EF-CODED-CHARACTER-SET external-format)))
         ;(octets-copied 0) ;(if null-terminate 1 0))
         (to-index -1)
         (needed-length (ash (+ (- end start) (if null-terminate 1 0))
                             (if (eq external-format :unicode) 1 0)))
         (mb-vector (cond ((and mb-vector (>= (length mb-vector) needed-length))
                           mb-vector)
                          ((or (not mb-vector) make-mb-vector?)
                           (make-array needed-length :element-type '(unsigned-byte 8)))
                                   ;:initial-element 0))
                          (t (error "Was given a vector of length ~A, ~but needed at least length ~A."
                                    (length mb-vector) needed-length)))))
    (declare (type (simple-array (unsigned-byte 8) (*)) mb-vector)
             (fixnum to-index))
    (do ((from-index start (1+ from-index)))
        ((>= from-index end))
      (declare (fixnum from-index))
      (let ((char (aref string from-index)))
        ;(if (char= char #\Newline)
        (cond ((eq external-format :unicode)
               (let ((char-code (char-code char)))
 #+little-endian (setf (aref mb-vector (incf to-index)) (ldb (byte 8 0) char-code)
                       (aref mb-vector (incf to-index)) (ldb (byte 8 8) char-code))
 #-little-endian (setf (aref mb-vector (incf to-index)) (ldb (byte 8 8) char-code)
                       (aref mb-vector (incf to-index)) (ldb (byte 8 0) char-code))))
              (code-page
               (setf (aref mb-vector (incf to-index))
                     (ef:char-external-code char code-page)))
              (t
               (setf (aref mb-vector (incf to-index)) (char-code char))) )))
    (when null-terminate
      (setf (aref mb-vector (incf to-index)) 0)
      (when (eq external-format :unicode)
        (setf (aref mb-vector (incf to-index)) 0)))
    (values mb-vector (1+ to-index))))

(defun octets-to-string (octets &key (start 0) end (external-format :default))
                                     ;string make-string? truncate
                                     ;(string-start 0) string-end
 ;;; Values: (1) a string,
  ;;	     (2) the number of characters copied,
  ;;	     (3) the number of octets used
  ;(declare (ignore string make-string? truncate string-start string-end
  (declare (type (simple-array (unsigned-byte 8) (*)) octets))
  (unless end (setq end (or (position 0 octets :start start) (length octets))))
  (when (eq external-format :default)
    (setq external-format #+win32 win32:*multibyte-code-page-ef*
                          #-win32 stream::*default-external-format*))
  (let* ((code-page (when (consp external-format)
                      (EF::EF-CODED-CHARACTER-SET external-format)))
         ;(octets-copied 0) ;(if null-terminate 1 0))
         (octet-length (- end start))
         (to-index -1)
         (string (make-string (if (eq external-format :unicode)
                                  (ash octet-length -1)
                                  octet-length)
                              :element-type 'lw:simple-char)))
    (declare (fixnum octet-length to-index) (type lw:simple-text-string string))
    (do ((from-index start (1+ from-index)))
        ((>= from-index end))
      (declare (fixnum from-index))
      (cond ((eq external-format :unicode)
             (let ((char-code 0))
                 (declare (fixnum char-code))
 #+little-endian (setf (ldb (byte 8 0) char-code) (aref octets from-index)
                       (ldb (byte 8 8) char-code) (aref octets (incf from-index)))
 #-little-endian (setf (ldb (byte 8 8) char-code) (aref octets from-index)
                       (ldb (byte 8 0) char-code) (aref octets (incf from-index)))
                 (setf (char string (incf to-index)) (code-char char-code))))
            (code-page
             (setf (char string (incf to-index))
                   (ef:find-external-char (the (unsigned-byte 8) (aref octets from-index))
                                          code-page)))
                     ;(code-char (win32::cp-to-unicode code-page nil
                     ;           (the (unsigned-byte 8) (aref octets from-index))))))
            (t
             (setf (char string (incf to-index)) (code-char (aref octets from-index)))) ))
    (values string (1+ to-index) octet-length)))

#|
(defun write-vector (sequence stream &key (start 0) end endian-swap)
  (declare (ignore endian-swap))
  (check-type sequence (or string (array (unsigned-byte 8) 1)
                           (array (signed-byte 8) 1)))
  (write-sequence sequence stream :start start :end end))

#|
(defun run-shell-command ()
  (with-open-stream (s (open-pipe "/bin/sh"
                                  :direction :io
                                  :buffered nil))
    (loop for var in environment
          do (format stream "~A=~A~%" (car var) (cdr var)))
|#
  

(provide 'acl-excl)
|#

#|
(in-package :sys)
(let ((*handle-warn-on-redefinition* :warn))
;      (*packages-for-warn-on-redefinition* nil))

  (defun command-line-arguments ()
    system:*line-arguments-list*)
  
  (defun command-line-argument (n)
    (nth n system:*line-arguments-list*))
  
  (defun reap-os-subprocess (&key (wait nil))
    (declare (ignore wait))
    nil)

  (export 'command-line-arguments)
  (export 'command-line-argument)
  (export 'reap-os-subprocess))

;; Franz uses the MSWINDOWS feature conditional in some of their code;
;; thus, under Windows, ACL-COMPAT should probably push MSWINDOWS
;; onto the *features* list when it detects the presence of WIN32
;; under Lispworks.
#+WIN32 (eval-when (:compile-toplevel :load-toplevel :execute)
          (pushnew :mswindows *features*))
|#


(defmacro mp::without-scheduling (&body forms)
  `(mp:without-preemption ,@forms))
